package de.mdelab.mlsdm.sdeimport.popup.actions;

import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.resources.IFile;
import org.eclipse.emf.common.util.URI;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IActionDelegate;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IWorkbenchPart;

import de.mdelab.workflow.ui.ExecuteWorkflowHandler;

public class SDEImportAction implements IObjectActionDelegate
{
	private static final String	INPUT_MODEL_URI	= "inputModelURI";
	private Shell				shell;
	private final List<IFile>	inputModelFiles;
	private final URI			workflowURI		= URI.createPlatformPluginURI("/de.mdelab.mlsdm.sdeImport/model/import.workflow", true);

	/**
	 * Constructor for Action1.
	 */
	public SDEImportAction()
	{
		super();
		this.inputModelFiles = new LinkedList<IFile>();
	}

	/**
	 * @see IObjectActionDelegate#setActivePart(IAction, IWorkbenchPart)
	 */
	@Override
	public void setActivePart(final IAction action, final IWorkbenchPart targetPart)
	{
		this.shell = targetPart.getSite().getShell();
	}

	/**
	 * @see IActionDelegate#run(IAction)
	 */
	@Override
	public void run(final IAction action)
	{
		for (final IFile iFile : this.inputModelFiles)
		{
			final ExecuteWorkflowHandler executeWorkflowAction = new ExecuteWorkflowHandler();

			final String inputModelURI = URI.createPlatformResourceURI(iFile.getFullPath().toString(), true).trimFileExtension().toString();

			final Map<String, String> propertyValues = new HashMap<String, String>();
			propertyValues.put(SDEImportAction.INPUT_MODEL_URI, inputModelURI);

			try {
				executeWorkflowAction.run( Collections.singletonList(this.workflowURI), this.shell, propertyValues );
			}
			catch (ExecutionException e) {
				e.printStackTrace();
				
				throw new RuntimeException( e );
			}
		}
	}

	/**
	 * @see IActionDelegate#selectionChanged(IAction, ISelection)
	 */
	@Override
	public void selectionChanged(final IAction action, final ISelection selection)
	{
		action.setEnabled(false);

		this.inputModelFiles.clear();

		if (!(selection instanceof IStructuredSelection) || selection.isEmpty())
		{
			return;
		}

		@SuppressWarnings("rawtypes")
		final Iterator it = ((IStructuredSelection) selection).iterator();

		while (it.hasNext())
		{
			final Object o = it.next();

			if (o instanceof IFile)
			{
				this.inputModelFiles.add((IFile) o);
			}
		}

		action.setEnabled(true);
	}

}
